'
' -----[ Title ]------------------------------------------------------
'
' File......: Multitask.BAS
' Purpose...: Sample of a pseudo Multitasking solution
' Author....: Mike Eitel
' Version...: 1.0AVR
' MCU.......: AVR
' Started...: 991002
' Updated...: 031703
'
' -----[ Program Description ]----------------------------------------
'
' This program implements the pseudo multitaslking in BASCOM for an universal
' purpous. Thsi solution is sucessfully running in various applications
' It uses fx. STK200 development kit.
'
' -----[ Disclaimer ]-----------------------------------------------------------
'
' This example is offered on an "AS IS" basis, no warranty expressed or implied.
' The programers disclaim liability of any damages associated with the use of
' the hardware or software described herein.
' You use it on your own risk. We are not able to provide any free support.
'
' Copyright (c) 2003 Mike Eitel, all rights reserved
'
' -----[ Revision History ]-----------------------------------------------------
'
' 011121 - Version 1.00AVR_LCD     Ver 1.00 Modbus slave for AVR>8515 Mike Eitel
' ......
' 030317 - Version 1.00AVR         Ver 1-00 Show multitask            Mike Eitel

' *****[ conditional TEST flag ]************************************************
Const Test = 1                               ' When 1 then testmode !
#if Test                                     ' ***** TEST MODE ***
#else                                        ' ***** TEST MODE ***
#endif                                       ' ***** TEST MODE ***
' *****[ conditional TEST flag ]************************************************

' -----[ Aliases ]--------------------------------------------------------------

$regfile = "8515def.dat"

' Define serial communication parameters
'Ucr = &B11011000                             ' Allow receiver & transmitter ISR
$baud = 4800

#if Test
$crystal = 4433618                           ' Frequency of unit
Ubrr = 57                                    ' For 4800 baud
'Ubrr = 28                                    ' For 9600 baud

#else                                        ' ***** TEST MODE ***
$crystal = 8000000                           ' Frequency of unit
Ubrr = 103                                   ' For 4800 baud
'Ubrr = 51                                    ' For 9600 baud
#endif                                       ' ***** TEST MODE ***

' -----[ LCD Configuration parameters  ]----------------------------------------
Config Lcd = 20 * 4                          ' DEFINE actual LCD configuration
Config Lcdmode = Port                        ' Use port mode and XRAM = None
Config Lcdpin = Pin , Db4 = Porta.4 , Db5 = Porta.5 , Db6 = Porta.6 , _
       Db7 = Porta.7 , E = Porta.3 , Rs = Porta.1       'dtr103
Cursor Off Noblink                           ' Dont disturb by cursor


' -----[ Constants ]------------------------------------------------------------
Const Initializing = &HFE                    ' Address to change Myaddress

' -----[ Variables ]------------------------------------------------------------
Dim Measure As Bit                           ' Measure it
Dim A As Integer                             ' Show something in line
Dim B As Integer                             ' Show something in line
Dim C As Integer                             ' Show something in line
Dim D As Integer                             ' Show something in line

Dim Taskswitcher_serial As Byte              ' Taskswitcher for serial comunication control
                                             ' 0 = Nothing recognized
                                             ' 1 = Node address dedected
                                             ' 2 = Valid command dedected
                                             ' 3 = Command execution
Dim Taskswitcher_p1 As Byte                  ' Taskswitcher for program I control
                                             ' Allways = Collect the I/O
                                             ' 2 = Collect the I/O
                                             ' 3 = Read the actual value
                                             ' 4 = Makes shure to do it only once
                                             ' 5 = Detect gap
 Dim Taskswitcher_p2 As Byte                 ' Taskswitcher for program II control
                                             ' 0 =
                                             ' 1 =
                                             ' 2 =
                                             ' 3 =
Dim Taskswitcher As Byte                     ' Taskswitcher status signalisation


' -----[ Initialization ]---------------------------------------------
'
' Define port parameters

  Disable Interrupts                         ' Stop the system timing

  ' Setup interrupts to defined startup

  Config Timer1 = Timer , Prescale = 256     ' Define update speed
  On Timer1 Timer1_isr                       ' Define ISR when timer 0 overflows                                         '
  Enable Timer1                              ' Allow I/O treating

  ' Define startup values after Powerup

  Taskswitcher_serial = 0                    ' Set to defined Startup value
  Taskswitcher_p1 = 0                        ' Set to defined Startup value
  Taskswitcher_p2 = 0                        ' Set to defined Startup value

  ' Show name and information of builders
  Cls                                        ' Clear the LCD display
  Upperline                                  ' Keyword for first line
  Lcd "Multitasking sample by Mike Eitel"    ' Wha is it
  Waitms 3000                                ' 3 seconds dalay
  Cls                                        ' Clear the LCD display

  Enable Interrupts                          ' Start the system timing from this line on

'------[ Program ]----------------------------------------------------
'
_com_z:
   Taskswitcher = Taskswitcher_serial        ' Three bits for serial communication
   Shift Taskswitcher , Left , 3
   Taskswitcher = Taskswitcher Or Taskswitcher_p1       ' Than two for I/O status
   Shift Taskswitcher , Left , 3
   Taskswitcher = Taskswitcher Or Taskswitcher_p2       ' Finaly two for controll inputs

   On Taskswitcher_serial Goto _com_0 , _com_1 , _com_2 , _com_3
   Taskswitcher_serial = 0                   ' Programmers fear of error statement
   Goto _p1_z

_com_0:                                      ' Search for this node address
   ' Startingpoint for telegrams
   Taskswitcher_serial = 1                   ' Node found -> telegram starts
   Goto _p1_z                                ' Give task switching a chance

_com_1:                                      ' Valid command dedect
   Taskswitcher_serial = 0                   ' No valid command -> clear try
   Goto _p1_z                                ' Give task switching a chance

_com_2:
   ' Get telegram into binary mode
   Taskswitcher_serial = 3                   ' CRC ok -> start encoding
   Goto _p1_z                                ' Give task switching a chance

_com_3:                                      ' Command execution
   Taskswitcher_serial = 0                   ' No valid command -> clear try
   Goto _p1_z                                ' programers fear of wrong loops

'------[ Here starts the Modbus independent part of the proram ]----------------
'-------------------------------------------------------------------------------
_p1_z:
   ' Collect the I/O
   On Taskswitcher_p1 Goto _p1_0 , _p1_1 , _p1_2 , _p1_3 , _p1_4
   Taskswitcher_p1 = 0                       ' Programmers fear of error statement
   Goto _p2_z

_p1_0:
   ' Read the actual value
   Taskswitcher_p1 = 1                       ' First part done
   Goto _p2_z                                ' Next is program 2

_p1_1:
   ' Makes shure to do it only once
   If Measure = 1 Then                       ' Make complete cycle
      Incr A
      Taskswitcher_p1 = 2                    ' Second part done
   Else
      Incr B
      Taskswitcher_p1 = 0                    ' Go back in loop
   End If
   Goto _p2_z                                ' Next is program 2

_p1_2:
   ' Detect if a new job is identified by a too long detected gap
   Incr B
   Taskswitcher_p1 = 3                       ' Third part done
   Goto _p2_z                                ' Next is program 2

_p1_3:
   ' One next marker is calculated in next program step
   Incr C
   Taskswitcher_p1 = 4                       ' Fourth part done
   Goto _p2_z                                ' Next is program 2

_p1_4:
   ' Calculate actual values and store in statistics
   Incr D
   Taskswitcher_p1 = 0                       ' Fifth part done
   Goto _p2_z                                ' Next is program 2

   ' Task p2 is handling the communication with the LCD
_p2_z:
   On Taskswitcher_p2 Goto _p2_0 , _p2_1 , _p2_2 , _p2_3 , _p2_4
   Taskswitcher_p2 = 0                       ' Programmers fear of error statement
   Goto _com_z                               ' Next is serial detection

_p2_0:
   Taskswitcher_p2 = 1                       ' First part done
   Goto _com_z                               ' Next is serial detection

_p2_1:
   Upperline                                 ' Keyword for first line
   Lcd "Line 1 = " ; A                       ' Last max distance
   Taskswitcher_p2 = 2                       ' Second part done
   Goto _com_z                               ' Next is serial detection

_p2_2:
   Lowerline                                 ' Keyword for first line
   Lcd "Line 2 = " ; B                       ' Last max distance
   Taskswitcher_p2 = 3                       ' Third part done
   Goto _com_z                               ' Next is serial detection

_p2_3:
   Thirdline                                 ' Keyword for first line
   Lcd "Line 3 = " ; C                       ' Last max distance
   Taskswitcher_p2 = 4                       ' Fourth part done
   Goto _com_z                               ' Next is serial detection

_p2_4:
   Fourthline                                ' Keyword for first line
   Lcd "Line 4 = " ; D                       ' Last max distance
   Taskswitcher_p2 = 0                       ' Fifth part done
   Goto _com_z                               ' Next is serial detection

End                                          ' End of the main program
' -----[ End ]--------------------------------------------------------


' -----[ Subroutines ]------------------------------------------------
'

' -----[ Interrupt routines ]------------------------------------------------

' Interrupt routine for preparing serial port
' On powerup it's treated once !
' This routine should not be needed but it only works with this routine.
Ude_isr:
   Ucr = &B11011000                          ' Number of character flag
Return


' Interrupt routine for reading serial input
Rec_isr:

Return


' Interrupt routine for sendig serial output
Send_isr:

Return


' Interrupt routine for multiplexing In / Outputs
Timer1_isr:
   Timer1 = 5000
   Measure = Not Measure                     '
Return